@@ -1,4 +1,5 @@
 #! /bin/sh
+# $Cambridge: exim/exim-src/scripts/Configure-os.c,v 1.1 2004/10/06 15:07:40 ph10 Exp $
 
 # Shell script to build os.c. There doesn't have to be an OS-specific os.c
 # file, but if there is, it gets copied at the start of os.c. The basic src
@@ -10,7 +11,7 @@ scripts=../scripts
 
 # First off, get the OS type, and check that there is a make file for it.
 
-os=`$scripts/os-type` || exit 1
+os=`$scripts/os-type -generic` || exit 1
 
 if	test ! -r ../OS/Makefile-$os
 then    echo ""
